
$(document).ready(function() {
	// Global for tracking the NEs to search for
	nesForSearch=[];
	// Global to track column names of visible/exportable columns
	visibleColumnIndex=[];
	// Add the release and version to the title
	addReleaseToTitle ();
	// Set up the basic table
	var aDataTable;
	var columnsInit;
	var rowCallbackInit=null;
	// Insert the header rows
	addTableHeader ('aTbl');
	// Set up the default columns 
	columnsInit = createColumnsData (aTblSrc.colData);
	//rowCallbackInit = createRowCallBack (aTblSrc.colData);
	//return;
	// Create the datatable object
	aDataTable = $('#aTbl').removeAttr('width').DataTable({
		data: aTblSrc.data,
		columns:columnsInit,
		"language": {
			"search": "Filter all records:"
		},
		// Hide the loading screen when the table is initialized
		"initComplete": function(settings, json) {
			$('#headerData').removeClass('hideMe');
			$('#loading').addClass('hideMe');
			$('#neSearch').removeClass('hideMe');
		},
		"searchDelay":400, // This can be reduced for small data sets
		"autoWidth":false,
		"deferRender":true,
		"ordering":true,
		"order":[[1, 'asc']],
		"dom": 'fBt', // Shows (in order): Filter, Buttons, Table
		"scrollY":400,	// Sets vertical table height
		//"scrollCollapse":false,
		"scrollX":true,
		"scroller":{rowHeight:35},
		"buttons":[]	// Sets empty buttons container
	});
	
	// Add customer event listeners 
	$('#aTbl').on( 'search.dt', function() {
		updateFilteredRows(aDataTable);
	}); 
	
	// Update specify column properties
	updateColData (aDataTable);
	
	// Add the pull-downs
	addPullDowns(aDataTable);
	
	// Setup the search for the column filters
	addSearchToFilters (aDataTable);
	
	// Change the default column sort
	moveSortListeners (aDataTable);
	
	// Add the button
	//addPrintButton (aDataTable);
	addExportButton (aDataTable);
	addExportVisibleButton (aDataTable);
	//addPdfButton (aDataTable);
	
	// Add the column toggles
	addColumnToggles (aDataTable);
	
	// Add additional search
	addCustomSearch(aDataTable);
	
	
	// Add custom filter
	//addCustomFilters (aDataTable);
	addCustomNEFilter (aDataTable);
	addACustomNEFilter (aDataTable);
	
	// Add the details function
	$('#aTbl tbody').on('click', 'td.nk-details', function () {
		var tr = $(this).closest('tr');
		var row = aDataTable.row( tr );
		if ( row.child.isShown() ) {
			// This row is already open - close it
			row.child.hide();
			tr.removeClass('shown');
		}
		else {
			// Open this row
			row.child( displayDetails(row.data()) ).show();
			tr.addClass('shown');
		}
	} );
	
	// Toggle the "about"
	$('.nk-toggleShow').on('click', function() {
		// Get the child span
		$(this).next('div').toggleClass('hideMe');
		// Toggle next div
		$('span:first', this).toggleClass('icon-plus icon-minus');
		// Toggle icon
		
	});
	
	// Add a listener on window resize
	addResizeListener (aDataTable)
	// Add x of y above table
	addToFrom (aDataTable);
	
	// Add the reset button
	//addResetButton(aDataTable);
});

var addReleaseToTitle = function () {
	// No footer download button for the stats
	var buttonContent;
	
	buttonContent = '<form method="get" action="data/statReleaseData.zip">'
		+ '<button class="downloadBtn" type="submit">Download stats</button>'
		+ '</form>';

	if (window.location.protocol === 'file:') {
		$('#header').removeClass('hideMe');
		addNavListener ();
		// We do not use the download button for local access
		//$('#footer').append(buttonContent);
		
	}
	else {
		// Add download button for server-based
		//$('#footer').append(buttonContent);
	}
};


var addNavListener = function () {
	$('a.nav').on('click', function(e) {
		e.preventDefault();
		// Get the name of the class to display
		var showInfo = $(this).attr('href');
		if (showInfo == 'splash') {
			$('#splash').removeClass('hideMe');
			$('#data').addClass('hideMe');
			$('#headerData').addClass('hideMe');
			$('#contact').addClass('hideMe');
		}
		if (showInfo == 'data') {
			$('#splash').addClass('hideMe');
			$('#data').removeClass('hideMe');
			$('#headerData').removeClass('hideMe');
			$('#contact').addClass('hideMe');
		}
		if (showInfo == 'contact') {
			$('#splash').addClass('hideMe');
			$('#data').addClass('hideMe');
			$('#headerData').addClass('hideMe');
			$('#contact').removeClass('hideMe');
		}
	});

}

var addSplashContent = function () {
	
}

var addResizeListener = function (aDataTable) {
	// Can set the debounce to a small amount if necessary
	// This forces the column headings to resize when the window is resized
	window.addEventListener('resize', debounce (function(){
	//	updateColData (aDataTable);
		// Allow 350 for header and filters
		// Allow minimum of 400 for table body
		// Get the current viewport
		var filler = 350;
		var tbody_min = 400;
		var h = $(window).height();
		var th = h - filler;
		if (h > (filler + tbody_min)) {
			$('div.dataTables_scrollBody').height( th );
		}
		else {
			$('div.dataTables_scrollBody').height( tbody_min );
		}
		
		aDataTable.draw();
	}, 10), true);

}

var addToFrom = function (aDataTable) {
	var info = aDataTable.page.info();
	$('div.dt-buttons').after('<div class="toFrom"><p class="rowCount"></p></div>');
	$('.rowCount').html('Displaying: ' + info.recordsDisplay + ' of ' + info.recordsTotal + ' records');
	
}

var updateFilterIcon = function (colName, filterValue) {
	$('a.toggle-vis').each(function() {
		if ($(this).text() == colName) {
			if (filterValue != '') {
				$(this).find('i').first().addClass('icon-filter');
			}
			else {
				$(this).find('i').first().removeClass('icon-filter');
			}
		}
		else {
			
		}
	});
}

var updateFilteredRows = function (aDataTable){
   var info = aDataTable.page.info();
   $('.rowCount').html('Displaying: ' + info.recordsDisplay + ' of ' + info.recordsTotal + ' records');
   // Alert content of search boxes
};

var displayDetails = function (rowData) {
	// Creates a two-column table containing the specified cell content
	var tblMsg='';
	tblMsg= '<table class="details" cellpadding:="0" cellspacing="0" border="0" style="padding-left:15px;" >'+
	'<tr>'+
	'<td>Stat xml name and display names:</td>'+
	'<td>'+ splitToList('display', rowData[0] + ' (' + rowData[1] + ')') +'</td>'+
	'</tr>'+
	'<tr>' +
	'<td>Policy xml name and display names:</td>'+
	'<td>'+ splitToList('display', rowData[3] + ' (' + rowData[4] + ')') +'</td>'+
	'</tr>'+
	'<tr>' +
	'<td>Description:</td>'+
	'<td>'+ rowData[2] +'</td>'+
	'</tr>'+
	
	'<tr>'+
	'<td>Supported products:</td>'+
	'<td>' + cleanProductData(rowData[9]) + '</td>'+
	'</tr>'+
	
	'<tr>'+
	'<td>Related performance stats:</td>'+
	'<td>'+ splitToList('display', rowData[5]) +'</td>'+
	'</tr>'+
	'<tr>'+
	'<td>Related realtime stats:</td>'+
	'<td>'+ splitToList('display', rowData[6]) +'</td>'+
	'</tr>'+
	'<tr>'+
	'<td>Counter stats:</td>'+
	'<td>'+ splitToList('display', rowData[7]) +'</td>'+
	'</tr>'+
	'<tr>'+
	'<td>Monitored classes:</td>'+
	'<td>'+ splitToList('display', rowData[8]) +'</td>'+
	'</tr>'+

	
	/*
	'<tr>'+
	'<td>Package and class:</td>'+
	'<td>'+rowData[6] + ', ' + rowData[7] + '</td>'+
	'</tr>'+
	
	'<td>Statistic Description:</td>'+
	'<td>'+rowData[2]+'</td>'+
	'</tr>'+

	'<tr>'+
	'<td>Monitored Classes:</td>'+
	'<td>'+ splitToList('display', rowData[10]) +'</td>'+
	'</tr>'+
	*/
	/*
	'<tr>'+
	'<td>Managed Object class:</td>'+
	'<td>'+rowData[4]+'</td>'+
	'</tr>'+
	'<tr>'+
	'<td>Probable cause:</td>'+
	'<td>'+rowData[5]+'</td>'+
	'</tr>'+
	'<tr>'+
	'<td>Specific cause:</td>'+
	'<td>'+rowData[6]+'</td>'+
	'</tr>'+
	'<tr>'+
	'<td>Menmonic:</td>'+
	'<td>'+rowData[7]+'</td>'+
	'</tr>'+
	'<tr>'+
	'<td>TL1 condition:</td>'+
	'<td>'+rowData[8]+'</td>'+
	'</tr>'+
	'<tr>'+
	'<td>Unique key:</td>'+
	'<td>'+rowData[9]+'</td>'+
	'</tr>'+
	'<tr>'+
	'<td>Event type:</td>'+
	'<td>'+rowData[10]+'</td>'+
	'</tr>'+
	'<tr>'+
	'<td>Alarm ID (type):</td>'+
	'<td>'+rowData[11]+'</td>'+
	'</tr>'+
	'<tr>'+
	'<td>Alarm ID (number):</td>'+
	'<td>'+rowData[12]+'</td>'+
	'</tr>'+
	'<tr>'+
	'<td>Default alarm severity:</td>'+
	'<td>'+rowData[13]+'</td>'+
	'</tr>'+
	'<tr>'+
	'<td>Default reporting:</td>'+
	'<td>'+rowData[14]+'</td>'+
	'</tr>'+
	'<tr>'+
	'<td>Service affecting:</td>'+
	'<td>'+rowData[15]+'</td>'+
	'</tr>'+
	'<tr>'+
	'<td>Repair action:</td>'+
	'<td>'+cleanData('display', rowData[16])+'</td>'+
	'</tr>'+
	*/
	
	'</table>';
	
	return (tblMsg);
}

var cleanProductData = function (data) {
	var releases,
		i, listSize,
		supported='';
	// Remove leading/trailing "-"
	data = data.replace(/(^-)|(-$)/g, "");
	if (data == '') {
		// What to use for no specifically supported releases
		return 'No specific product';
	}
	releases = data.split('-');
	listSize = releases.length;
	for (i=0; i<listSize; i++) {
		// Check if this is part of a search
		if (nesForSearch.indexOf(releases[i]) != -1) {
	//	if (true) {
			supported += '<b>' + aTblSrc.products[releases[i]] + '</b>';
		}
		else {
			supported += aTblSrc.products[releases[i]];
		}
		if (i<listSize - 1) {
			supported += ', ';
		}
		// Arbitrary br every 5 objects
		if (i%5 == 0 && i > 0) {
			supported += '<br>';
		}
	}
	return supported;
}

var getSupportedObjects = function (data) {
	var releases,
		i, listSize,
		supported='';
	// Remove leading/trailing "-"
	data = data.replace(/(^-)|(-$)/g, "");
	if (data == '') {
		// What to use for no specifically supported releases
		return 'No specific releases';
	}
	releases = data.split('-');
	listSize = releases.length;
	for (i=0; i<listSize; i++) {
		supported += aTblSrc.products[releases[i]];
		if (i<listSize - 1) {
			supported += ', ';
		}
	}
	return supported
}

var createColumnsData = function (colData) {
	// Creates the constructor used by datatables
	var aCol,
		i,
		listSize,
		colOffset=0,	// If any columns are toggle-only, the data offsets need to be adjusted
						// (toggle columns are not included in the source data)
		cols = [];
	
	listSize = colData.length;
	for (i=0; i<listSize; i++) {
		aCol = {};
		// Check for the "details" toggle
		if (colData[i].isToggle==true) {
			aCol.data = null;
			aCol.defaultContent='';
			aCol.sortable=false;
			aCol.className = 'nk-details';
			colOffset+=1;
		}
		else {
			aCol.data=i-colOffset;
		}
		// Check for default visibility
		if (colData[i].hasOwnProperty('initVisible')) {
			if (colData[i].initVisible == false) {
				aCol.visible = false;
			}
		}
		if (colData[i].asList == true) {
			aCol.data=null;
			aCol.render = function (data, type, row, meta){return splitToList(type, row[meta.col-colOffset])}
		}
		if (colData[i].render == true) {
			aCol.data=null;
			aCol.render = function (data, type, row, meta){return cleanData(type, row[meta.col-colOffset])}
		}
		if (colData[i].hasOwnProperty('className')) {
			aCol.className = colData[i].className;
		}
		// Add sort - required if any columns are hidden initially
		aCol.sortable=true;
		aCol.width=colData[i]['width'];
		cols.push(aCol);
	}
	//alert (JSON.stringify(cols));
	return cols;
}

var splitToList = function (type, data) {
	// Simple reformating for some of the large fields.
	var cellData = '',
		parts=[],
		items=[],
		i,
		listSize;
	if (typeof (data) !== 'string') {
		return data;
	}
	
	// For exporting to csv, use the plain source data
	if (type == 'export') {
		return data;
	}
	if (data == "") {
		return data;
	}
	parts = data.split('; ');
	listSize = parts.length;
	items.push('<ul class="dtList">');
	for (i=0; i<listSize; i++) {
		items.push('<li>' + parts[i] + '</li>');
	}
	items.push('</ul>');
	return items.join('');
}

var cleanData = function (type, data) {
	// Modest formatting on text strings
	var cellData = '';
	var parts=[];
	if (typeof (data) !== 'string') {
		return data;
	}
	// For exporting to csv, use the plain source data
	if (type == 'export') {
		return data;
	}
	parts = data.split('\\n');
	if (parts.length>1) {
		return '<p>' + parts.join('</p><p>') + '</p>';
	}
	return data;
}



var applyCurrentSearch = function (aDataTable, resetSearch) {
	// Reads current search fields and applies filters
	// resetSearch will reset filters (i.e., display all rows)
	
	// Get the current text used in the filter fields
	var filterText = [];
	
	// Reset the search
	if (typeof(resetSearch) === 'boolean' && resetSearch) {
		aDataTable.columns().search('');
	}
	aDataTable.columns().every( function ( colIdx ) {
		//alert ($('input', aDataTable.column( colIdx ).header()).val);
		var headerText = $('input', aDataTable.column( colIdx ).header()).val();
		if (headerText != '' && typeof headerText === 'string') {
			aDataTable
				.column( colIdx )
				.search( headerText );
		}
		filterText.push(headerText);
	});
	aDataTable.draw();
};


var addCustomSearch = function (aDataTable) {
	var s, searchS;
	// Add input box
	$('#custSearch').append('<label>Negative filter:&#160;<input type="text" id="custInput"></label>');
	$('#custInput').on('keyup input', debounce (function(){
		
		s = $('#custInput').val();
		if (s == '') {
	//		aDataTable.columns().search('').draw();
		//	aDataTable.draw();
		// Run search using existing input
			applyCurrentSearch(aDataTable, true);
			return;
		}
		else {
			searchS = '^((?!' + s + ').)*$';
			aDataTable.columns().search(searchS, true).draw();
			applyCurrentSearch(aDataTable, false);
		}
	}, 500));
};


var addColumnToggles = function (aDataTable) {
	// Add links
	var i, listSize,
		togglePara,
		className,
		isToggleable,
		paraContent='<h3>Toggle columns:&#160;</h3>';
	togglePara = $('#tblToggle');
	listSize = aTblSrc.colData.length;
	
	for (i=0; i<listSize; i++) {
		// Check if this column should be toggleable
		isToggleable = true;
		className = '';
		if (aTblSrc.colData[i].isToggleable == false) {
			isToggleable = false;
		}
		// Check if this column should be displayed intitially
		if (aTblSrc.colData[i].initVisible == false) {
			className= ' colHidden';
		}
		if (isToggleable) {
			paraContent+= '<a class="toggle-vis' + className + '" data-column="' + i + '">' + aTblSrc.colData[i].title + '<i></i></a>&#160;';
		}
	}
	
	togglePara.html(paraContent);
	// Setup listener
	$('a.toggle-vis').on('click', function(e) {
		e.preventDefault();
		// Get the column API object
		var column = aDataTable.column($(this).attr('data-column'));
		// Get the column status
		// Toggle visibility
		column.visible(!column.visible());
		// Toggle the "hidden" class
		$(this).toggleClass('colHidden');
		updateColData(aDataTable);
		// Update sort listeners
		moveSortListeners (aDataTable);
	});

}

var updateColData = function (aDataTable) {
	// Updates/overwrites the datatables object with specific
	// column information
	// Checks for hidden columns when determinging the widths
	var idx;
	var colWidths=[];	// Array of widths of the visible columns
	// Set widths in datatables object
	aDataTable.columns().every( function ( colIdx ) {
		if (this.visible()) {
			var column = this;
			column.width = aTblSrc.colData[colIdx].width;
			colWidths.push(aTblSrc.colData[colIdx].width)
		}
		
	});
	// Set widths to the visible columns tables
	$('.tbl-marker').find('th, thead td').each(function(){
		idx=this.cellIndex;
		$(this).css({'width':colWidths[idx]});
	});	
};

var addResetButton = function (aDataTable) {
	// Add button to div
	aDataTable.button().add( 0, {
		text:'Reset search',
		action: function (e, dt, button, config) {
			alert ('Hello');
			var info = '';
			$('.nk-input').each(function() {
				alert (this.placeholder);
				//info += (this.value) + ', ';
			});
			alert (info);
		}

	});
	// Append the button to the div containing the filter
	aDataTable.buttons().container()
		.appendTo('#dataTables_filter');
}

var addExportButton = function (aDataTable) {
	// This adds an "export to csv" function.
	// This version uses a dynamic filenaming function.
	// It's probably better to use fixed names (possbibly including a date)
	aDataTable.button().add( 0, {
		extend:'csv', 
		text:'CSV - Export all',
		title:"CSVAll",
		action: function (e, dt, button, config) {
			config.title = "NFM-P_ACC_Stats_AllColData";

			// This checks whether the HTML5 or flash version needs
			// to be called, so it's not needed if the button you are using 
			// doesn't have that distinction
			if ($.fn.dataTable.ext.buttons.csvHtml5.available( dt, config )) {
				$.fn.dataTable.ext.buttons.csvHtml5.action(e, dt, button, config);
			}
			else {
				$.fn.dataTable.ext.buttons.csvFlash.action(e, dt, button, config);
			}
		},
		exportOptions:{
			orthogonal: 'export',
			format:{
				header: function (data, column, row) {
					return (aTblSrc.colData[column].title)
				},
				body: function (data, row, column, node) {
					// Explcitly test for the NE Releases column
					if (column == 11) {
						return getSupportedObjects (data);
					}
					return data;
				}
			},
			
			columns :  function(idx, data, node) {
				// Hide the first column (the open/close)
				if (idx == 0) {
					return false;
				}
				return true;
			}
			
		}
	});
	// Append the button to the div containing the filter
	aDataTable.buttons().container()
		.appendTo('#dataTables_filter');
}


var exportColumsForCSV = function (idx, data, node, aDataTable) {
	// Hide the first column (the open/close)
	
	if (idx == 0) {
		return false;
	}
	if (idx==2) {
		return true;
	}
	alert (idx);
	
	return (aDataTable.column(idx).visible());
}

var addExportVisibleButton = function (aDataTable) {
	// This adds an "export to csv" function.
	// This version uses a dynamic filenaming function.
	// It's probably better to use fixed names (possbibly including a date)
	aDataTable.button().add( 0, {
		extend:'csv', 
		text:'CSV - Export visible',
		title:"CSVVisible",
		"vList":[],
		action: function (e, dt, button, config) {
			config.title = "NFM-P_ACC_Stats_VisibleColData";
			// Map visible rows
			updateVisibleColumnNames (aDataTable);
			// This checks whether the HTML5 or flash version needs
			// to be called, so it's not needed if the button you are using 
			// doesn't have that distinction
			if ($.fn.dataTable.ext.buttons.csvHtml5.available( dt, config )) {
				$.fn.dataTable.ext.buttons.csvHtml5.action(e, dt, button, config);
			}
			else {
				$.fn.dataTable.ext.buttons.csvFlash.action(e, dt, button, config);
			}
		},
		exportOptions:{
			orthogonal: 'export',
			format:{
				header: function (data, column, row) {
					return (aTblSrc.colData[column].title)
				},
				body: function (data, row, column, node) {
					if (visibleColumnIndex[column] == 'Supported Products') {
						return getSupportedObjects (data);
					}
					return data;
				}
			},
			// Control which columns will appear in the output
			columns :  function(idx, data, node) {
				// Hide the first column (the open/close)
				if (idx == 0) {
					return false;
				}
				// Always output the supported products
				if (aTblSrc.colData[idx].title == 'Supported Products') {
					return true;
				}
				return (aDataTable.column(idx).visible());
			}
			
		}
	});
	// Append the button to the div containing the filter
	aDataTable.buttons().container()
		.appendTo('#dataTables_filter');
}


var updateVisibleColumnNames = function (aDataTable) {
	// If any columns required test modification for display or export,
	// they will need to be explicitly identified using the column index.
	// The column index may change, depending on which columns are visible
	var nameList = [],
		i, listSize,
		j=0;
		
	// Reset global list of column name indexes
	visibleColumnIndex.length=0;
	
	listSize = aDataTable.columns().nodes().length;
	for (i=0; i<listSize; i++) {
		if (aTblSrc.colData[i].isToggle == true) {
			// Skip the "defaults" column
		}
		// If column is currently visible, record the index
		else if (aDataTable.column(i).visible()== true) {
			visibleColumnIndex.push(aTblSrc.colData[i].title);
			j++;
		}
		// Always record the supported NEs
		else if (aTblSrc.colData[i].title == 'Supported Products') {
			visibleColumnIndex.push(aTblSrc.colData[i].title);
			j++;
		}
	}
	
}

var addCustomNEFilter = function (aDataTable) {
	var i, j,
		listSize, jsize,
		html='';
	html += '<h3 id="showNEs" class="nk-col"><span class="icon-plus"></span>Filter by Supported Product<span></span></h3>';
	html += '<div id="neCBs" class="hideMe">';
	html += '<button id="neSearchBtn" class="dt-button">Search by NEs</button>';
	html += '<button id="clearNEs" class="dt-button">Clear selections</button>';
	
	listSize = aTblSrc.products.length;
	for (i=0; i<listSize; i++) {
		// One list per product
		html += '<div class="supProduct">';
		html += '<span class="product">' + aTblSrc.products[i] + '</span> ';
		html += '<label><input type="checkbox" name="checkbox" value="' + i + '">' + '</label>';
		// Add the checkbox
		
		// Disable. This may be restored if we need to include multiple releases
		/*
		jsize = aTblSrc.products[i]['releases'].length;
		for (j=0; j<jsize; j++) {
			release = aTblSrc.supported[i]['releases'][j]
			html += '<label><input type="checkbox" name="checkbox" value="' + release[1] + '">' + release[0] + '</label>';
		}
		*/
		html+= '</div>';
	}
	html += '</div>';
	$('#neSearch').append(html);
	
	
	// Add the on click toggle of NEs
	$('#showNEs').on('click', function () {
		$('#neCBs').toggleClass('hideMe');
		// Change the icon
		$('span:first', this).toggleClass('icon-plus icon-minus');
		
	});
	// Add the onclick search
	$('#neSearchBtn').on('click', function () {
		filterCustomNEs (aDataTable);
		aDataTable.draw();
		// Shut the display
		$('#neCBs').toggleClass('hideMe');
		$('#showNEs span:first').toggleClass('icon-plus icon-minus');
		// Add filter icon
		if (nesForSearch.length > 0) {
			$('#showNEs span').eq(1).addClass('icon-filter');
		}
		else {
			$('#showNEs span:nth-of-type(2)').removeClass('icon-filter');
		}
	});
	
	$('#clearNEs').on('click', function() {
		clearSelectedNEs (aDataTable);
	});
}



var addACustomNEFilter = function (aDataTable) {
	// Custom filter for NEs
	// NE column is 12 (includes the toggle "Show" column
	var neColumn = 10;
	$.fn.dataTable.ext.search.push(
		function( settings, data, dataIndex ) {
			var i, listSize;
			/*
			$('#neSearch input').each(function (i) {
				var item = $(this);
				if (item.prop('checked')) {
					nes.push(item.prop('value'));
				}
			});
			*/
			if (nesForSearch.length > 0) {
				listSize = nesForSearch.length;
				for (i=0; i<listSize; i++) {
					// Needs index to product column
					if (data[neColumn].indexOf('-' + nesForSearch[i] + '-') != -1) {
						return true;
					}
				}
				return false;
			}
			return true;
		}
	);	
}


var clearSelectedNEs = function (aDataTable) {
	nesForSearch = [];
	$('#neSearch input').each(function (i) {
		var item = $(this);
		item.prop('checked', false);
	});
	
};

var filterCustomNEs = function (aDataTable) {
	// Get list of NEs to search for
	nesForSearch = [];
	$('#neSearch input').each(function (i) {
		var item = $(this);
		if (item.prop('checked')) {
			nesForSearch.push(item.prop('value'));
		}
	});
	/*
	if (searchFor.length > 0) {
		// Add search filter
		alert (searchFor);
		aDataTable.column(2).search('-' + searchFor.join('-') + '-').draw();
		
		// Remove search 
	}
	*/
}

var addCustomFilters = function (aDataTable) {
	// Custom search for the supported NEs
	// NE column is 12 (includes the toggle "Show" column
	var neColumn = 10;
	$('#neSearch').append('<label>Supported NEs:&#160;<input type="text" id="neFilter"></label>');
	// Add the listener$('#)
	$('#neFilter').on('keyup input', function() {
		aDataTable.draw();
	});
	$.fn.dataTable.ext.search.push(
		function( settings, data, dataIndex ) {
			// Get the value of the supported NEs column
			var neVal = data[neColumn]; // use data for the products column
			// Get the search mask
			var searchNEs = $('#neFilter').val();
			alert (searchNEs);
			if (searchNEs == '') {
				return true;
			}
			searchNEs = searchNEs.split(' ');
			var i, listSize;
			var matches=0;
			listSize = searchNEs.length;
			for (i=0; i<listSize; i++) {
				if (neVal.indexOf('-' + searchNEs[i] + '-') != -1) {
					//return true;
					matches+=1;
				}
			}
			if (matches == listSize) {
				return true
			}
			return false;
		}
	);	
}

var addPullDowns = function (aDataTable) {
	// Sets pull-down content
	var i, listSize, val;
	
	aDataTable.columns().every( function (idx) {
		if (aTblSrc.colData[idx].hasOwnProperty('nk-pull')) {
			var column = this;
			var select = $('<select><option value=""></option></select>')
				.appendTo( $(column.header()).empty() )
				.on( 'change', function () {
					var val = $.fn.dataTable.util.escapeRegex(
						$(this).val()
					);

					column
						.search( val ? '^'+val+'$' : '', true, false )
						.draw();
				} );
			listSize = aTblSrc.colData[idx]['nk-pull'].length;
			for (i=0; i<listSize; i++) {
				val = aTblSrc.colData[idx]['nk-pull'][i];
				select.append( '<option value="'+val+'">'+val+'</option>');
			}
			/*
			For dynamic calculation:
			column.data().unique().sort().each( function ( d, j ) {
				select.append( '<option value="'+d+'">'+d+'</option>' )
			} );
			*/
		}
	} );
};

var addPdfButton = function (aDataTable) {
	aDataTable.button().add( 0, {
		extend:'pdf', title:'MyTable',
		text:'Save as PDF',
		exportOptions:{
			format:{
				header: function (data, column, row) {
					return (aTblSrc.colData[column].title)
				}
			},
			/*
			columns :  function(idx, data, node) {
				// Hide the first column (the open/close)
				if (idx == 0) {
					return false;
				}
				return true;
			}
			*/
		}
	});
	// Append the button to the div containing the filter
	aDataTable.buttons().container()
		.appendTo('#dataTables_filter');
	
};

var addPrintButton = function (aDataTable) {
	aDataTable.button().add( 0, {
		extend: 'print',
		title:'MyPrint',
		customize: function ( win ) {
			$(win.document.body)
				.css( 'font-size', '10pt' )
				.prepend(
					'<img src="http://datatables.net/media/images/logo-fade.png" style="position:absolute; top:0; left:0;" />'
				);

			$(win.document.body).find( 'table' )
				.addClass( 'compact' )
				.css( 'font-size', 'inherit' );
		}
	});
	// Append the button to the div containing the filter
	aDataTable.buttons().container()
		.appendTo('#dataTables_filter');
	
}

var addSearchToFilters = function (aDataTable) {
	// Add the separate column filters, with a small debounce
	aDataTable.columns().every( function ( colIdx ) {
		// Small debounce on search filter
		$( 'input', aDataTable.column( colIdx ).header() ).on( 'keyup input', debounce(function () {
			aDataTable
				.column( colIdx )
				.search( this.value )
				.draw();
			// Add icon
			updateFilterIcon(this.placeholder, this.value);
		}, 500));
	} );
}

var updateSortListeners = function (aDataTable) {
	var i, listSize;
	// Disable current listeners
	$('td').off('click.DT');
	aDataTable.off('order.dt');
	// Add listeners to the columns
	listSize = aTblSrc.colData.length;
	for (i=0; i<listSize; i++) {
	//	aDataTable.order.listener($('#aTbl-' + i), i);
	}

}

moveSortListeners = function (aDataTable) {
	var i, listSize;
	// When the column filters are moved to the table header, this affects whether clicking
	// on the header will resort the columns.
	// Disable existing sort (probably on the filter row)
	$('td').off('click.DT');
	// Iterate through the th elements in the first row of the thead
	/*
	$('#aTbl').find('thead td').each(function(colIdx) {
		// Add the sort listener to this cell
		aDataTable.order.listener($('#aTbl-' + colIdx), colIdx);
	});
	*/
	listSize = aTblSrc.colData.length;
	for (i=0; i<listSize; i++) {
		aDataTable.order.listener($('#aTbl-' + i), i);
	}

}

var addTableHeader = function (tblId) {
	// Sets up the two header rows. One has the titles,
	// the other is used for the column filters
	// Adds ids to the first row. This allows the sort event listener
	// to be added.
	var aTbl,
		i,
		listSize,
		data='<thead>';
		
	// Get the table element
	aTbl = $('#' + tblId);
	data+='<tr>';
	// Add the column title row
	listSize = aTblSrc.colData.length;
	for (i=0; i<listSize; i++) {
		data += '<th id="' + tblId + '-' + i + '" class="nk-col">' + aTblSrc.colData[i].title + '</th>';
	}
	data+='</tr>';
	// Add the column filter row
	
	data+='<tr>';
	for (i=0; i<listSize; i++) {
		if (aTblSrc.colData[i].isToggle==true) {
			data+='<td></td>';
		}
		else {
		data += '<td><input type="text" class="nk-input" placeholder="' + aTblSrc.colData[i].title +'" /></td>';
		}
	}
	data+='</tr>';
	
	data+='</thead>';
	aTbl.append(data);	
}


//http://davidwalsh.name/javascript-debounce-function
function debounce(func, wait, immediate) {
	var timeout;
	return function() {
		var context = this, args = arguments;
		var later = function() {
			timeout = null;
			if (!immediate) func.apply(context, args);
		};
		var callNow = immediate && !timeout;
		clearTimeout(timeout);
		timeout = setTimeout(later, wait);
		if (callNow) func.apply(context, args);
	};
};	

	
	
	
